﻿using SpanTTest.Common;
using System;
using System.Collections.Generic;
using System.Text;

namespace SpanTTest
{
    class Program
    {

        static int count = 1000000;

        static string str = "希特勒于1935年3月废除《凡尔赛条约》后，德国军方胃口大增，对尚在定型中的III号和IV号坦克不再满足了。因为它们的装甲防护相对国外坦克不占优势，其平均装甲厚度为30毫米，而英、法坦克的平均装甲厚度为40～60毫米，开发新式重型坦克的工作势在必行。";

        static string subStr = "《凡尔赛条约》";

        static string splitStr = "坦克";

        static void Main(string[] args)
        {
            TestIndexString();

            TestSubString();

            TestSplitString();

            TestReplaceString();

            TestContainString();

            TestCompareString();

            TestLinkString();

            Console.WriteLine("测试已完成!");
            Console.ReadLine();
        }

        static void TestIndexString()
        {
            Console.WriteLine("回车开始 IndexOf 方法测试!");
            Console.ReadLine();

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    str.IndexOf(subStr);
                }
            });

            Console.WriteLine($"String.IndexOf {count} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var subStrSpan = subStr.AsSpan();

                for (int i = 0; i < count; i++)
                {
                    strSpan.IndexOf(subStrSpan);
                }
            });

            Console.WriteLine($"Span<T>.IndexOf {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }

        static void TestSubString()
        {
            Console.WriteLine("回车开始 Substring 方法测试!");
            Console.ReadLine();

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    str.Substring(0, str.IndexOf(subStr));
                }
            });

            Console.WriteLine($"String.Substring {count} 次用时：{t1} 毫秒");


            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var subStrSpan = subStr.AsSpan();

                var index = strSpan.IndexOf(subStrSpan);

                for (int i = 0; i < count; i++)
                {
                    strSpan.Slice(0, strSpan.IndexOf(subStrSpan));
                }
            });

            Console.WriteLine($"Span<T>.Substring {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }

        static void TestSplitString()
        {
            Console.WriteLine("回车开始 Split 方法测试!");
            Console.ReadLine();


            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    var arr = str.Split(new string[] { splitStr }, StringSplitOptions.None);
                }
            });

            Console.WriteLine($"String.Split {count} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var splitSapn = splitStr.AsSpan();

                int m = 0, n = 0;

                for (int i = 0; i < count; i++)
                {
                    List<string> arr = new List<string>();

                    while (true)
                    {
                        m = n;
                        n = strSpan.IndexOf(splitSapn);
                        if (n > -1)
                        {
                            arr.Add(strSpan.Slice(0, n).ToString());
                            strSpan = strSpan.Slice(n + splitSapn.Length);
                        }
                        else
                        {
                            break;
                        }
                    }

                }
            });

            Console.WriteLine($"Span<T>.Split {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }

        static void TestReplaceString()
        {
            Console.WriteLine("回车开始 Replace 方法测试!");
            Console.ReadLine();

            var replaceStr = "飞机";

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    str.Replace(splitStr, replaceStr);
                }
            });

            Console.WriteLine($"String.Replace {count} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var splitSapn = splitStr.AsSpan();

                int m = 0, n = 0;

                for (int i = 0; i < count; i++)
                {
                    List<string> arr = new List<string>();

                    while (true)
                    {
                        m = n;
                        n = strSpan.IndexOf(splitSapn);
                        if (n > -1)
                        {
                            arr.Add(strSpan.Slice(0, n).ToString());
                            strSpan = strSpan.Slice(n + splitSapn.Length);
                        }
                        else
                        {
                            break;
                        }
                    }
                    string.Join(replaceStr, arr);
                }
            });

            Console.WriteLine($"Span<T>.Replace {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }                       
        
        static void TestContainString()
        {
            Console.WriteLine("回车开始 Contains 方法测试!");
            Console.ReadLine();

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    str.Contains(subStr);
                }
            });

            Console.WriteLine($"string.Contains {count} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var subStrSpan = subStr.AsSpan();

                for (int i = 0; i < count; i++)
                {
                    strSpan.Contains(subStrSpan, StringComparison.CurrentCulture);
                }
            });

            Console.WriteLine($"Span<T>.Contains {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }

        static void TestCompareString()
        {
            Console.WriteLine("回车开始 Compare 方法测试!");
            Console.ReadLine();

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < count; i++)
                {
                    string.Compare(str, subStr, true);
                }
            });

            Console.WriteLine($"string.Compare {count} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                var strSpan = str.AsSpan();

                var subStrSpan = subStr.AsSpan();

                for (int i = 0; i < count; i++)
                {
                    strSpan.Contains(subStrSpan, StringComparison.CurrentCultureIgnoreCase);
                }
            });

            Console.WriteLine($"Span<T>.Contains {count} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }

        static void TestLinkString()
        {
            Console.WriteLine("回车开始 link 方法测试!");
            Console.ReadLine();            

            var testCount = 100000;

            var lstr = string.Empty;

            var t1 = StopWacherHelper.DoTimer(() =>
            {
                for (int i = 0; i < testCount; i++)
                {
                    lstr += splitStr;
                }
            });

            Console.WriteLine($"string.link {testCount} 次用时：{t1} 毫秒");

            var t2 = StopWacherHelper.DoTimer(() =>
            {
                List<byte> list = new List<byte>();

                for (int i = 0; i < testCount; i++)
                {
                    list.AddRange(Encoding.UTF8.GetBytes(splitStr));
                }

                Encoding.UTF8.GetString(list.ToArray());

            });

            Console.WriteLine($"Encoding.GetString {testCount} 次用时：{t2} 毫秒");
            Console.WriteLine();
        }
    }
}
